from __future__ import annotations

import time
from http import HTTPStatus
from pathlib import Path
from unittest import mock
from unittest import skipIf

import django
from django.conf import settings
from django.http import StreamingHttpResponse
from django.middleware.gzip import GZipMiddleware
from django.test import override_settings
from django.test import RequestFactory
from django.test import SimpleTestCase

import django_browser_reload
from django_browser_reload import views
from tests.compat import aiter
from tests.compat import anext


class OnAutoreloadStartedTests(SimpleTestCase):
    def test_success(self):
        calls: list[tuple[Path, str]] = []

        class FakeReloader:
            def watch_dir(self, directory, glob):
                calls.append((directory, glob))

        reloader = FakeReloader()

        views.on_autoreload_started(sender=reloader)

        assert calls == [
            (settings.BASE_DIR / "templates" / "jinja", "**/*"),
            (settings.BASE_DIR / "static", "**/*"),
            (Path(django_browser_reload.__file__).parent / "static", "**/*"),
        ]


class OnFileChangedTests(SimpleTestCase):
    def test_ignored(self):
        views.on_file_changed(file_path=Path("/tmp/nothing"))

        time.sleep(views.RELOAD_DEBOUNCE_TIME * 1.1)
        assert not views.should_reload_event.is_set()

    def test_django_template(self):
        path = settings.BASE_DIR / "templates" / "django" / "example.html"

        result = views.on_file_changed(file_path=path)

        time.sleep(views.RELOAD_DEBOUNCE_TIME * 1.1)
        assert result is True
        assert views.should_reload_event.is_set()
        views.should_reload_event.clear()

    def test_jinja_template(self):
        path = settings.BASE_DIR / "templates" / "jinja" / "example.html"

        result = views.on_file_changed(file_path=path)

        time.sleep(views.RELOAD_DEBOUNCE_TIME * 1.1)
        assert result is True
        assert views.should_reload_event.is_set()
        views.should_reload_event.clear()

    def test_static_asset(self):
        path = settings.BASE_DIR / "static" / "example.css"

        result = views.on_file_changed(file_path=path)

        time.sleep(views.RELOAD_DEBOUNCE_TIME * 1.1)
        assert result is True
        assert views.should_reload_event.is_set()
        views.should_reload_event.clear()


@override_settings(DEBUG=True)
class EventsTests(SimpleTestCase):
    @override_settings(DEBUG=False)
    def test_fail_not_debug(self):
        response = self.client.get("/__reload__/events/")

        assert response.status_code == HTTPStatus.NOT_FOUND

    def test_fail_not_accepted(self):
        response = self.client.get("/__reload__/events/", HTTP_ACCEPT="text/html")

        assert response.status_code == HTTPStatus.NOT_ACCEPTABLE

    def test_success_ping(self):
        response = self.client.get("/__reload__/events/")
        assert isinstance(response, StreamingHttpResponse)

        assert response.status_code == HTTPStatus.OK
        assert response["Content-Type"] == "text/event-stream"
        response_iterable = iter(response)
        event = next(response_iterable)
        assert event == (
            b'data: {"type": "ping", "versionId": "'
            + views.version_id.encode()
            + b'"}\n\n'
        )

    @mock.patch.object(views, "PING_DELAY", 0.001)
    def test_success_ping_twice(self):
        response = self.client.get("/__reload__/events/")
        assert isinstance(response, StreamingHttpResponse)

        assert response.status_code == HTTPStatus.OK
        assert response["Content-Type"] == "text/event-stream"
        response_iterable = iter(response)
        event1 = next(response_iterable)
        event2 = next(response_iterable)
        assert event1 == event2

    def test_success_template_change(self):
        response = self.client.get("/__reload__/events/")
        assert isinstance(response, StreamingHttpResponse)
        views.should_reload_event.set()

        assert response.status_code == HTTPStatus.OK
        assert response["Content-Type"] == "text/event-stream"
        response_iterable = iter(response)
        # Skip version ID message
        next(response_iterable)
        event = next(response_iterable)
        assert event == b'data: {"type": "reload"}\n\n'
        assert not views.should_reload_event.is_set()

    def test_success_template_change_with_gzip(self):
        middleware = GZipMiddleware(views.events)
        request = RequestFactory(HTTP_ACCEPT_ENCODING="gzip").get("/")
        response = middleware(request)
        assert isinstance(response, StreamingHttpResponse)
        views.should_reload_event.set()

        assert response.status_code == HTTPStatus.OK
        assert response["Content-Type"] == "text/event-stream"
        assert response["Content-Encoding"] == ""
        response_iterable = iter(response)
        # Skip version ID message
        next(response_iterable)
        event = next(response_iterable)
        assert event == b'data: {"type": "reload"}\n\n'
        assert not views.should_reload_event.is_set()


django_4_2_plus = skipIf(django.VERSION < (4, 2), "Requires Django 4.2+")


@override_settings(DEBUG=True)
class AsyncEventsTests(SimpleTestCase):
    @override_settings(DEBUG=False)
    async def test_fail_not_debug(self):
        response = await self.async_client.get("/__reload__/events/")

        assert response.status_code == HTTPStatus.NOT_FOUND

    async def test_fail_not_accepted(self):
        response = await self.async_client.get(
            "/__reload__/events/", ACCEPT="text/html"
        )

        assert response.status_code == HTTPStatus.NOT_ACCEPTABLE

    @django_4_2_plus
    async def test_success_ping(self):
        response = await self.async_client.get("/__reload__/events/")
        assert isinstance(response, StreamingHttpResponse)

        assert response.status_code == HTTPStatus.OK
        assert response["Content-Type"] == "text/event-stream"

        event = await anext(aiter(response))
        assert event == (
            b'data: {"type": "ping", "versionId": "'
            + views.version_id.encode()
            + b'"}\n\n'
        )

    @django_4_2_plus
    @mock.patch.object(views, "PING_DELAY", 0.001)
    async def test_success_ping_twice(self):
        response = await self.async_client.get("/__reload__/events/")
        assert isinstance(response, StreamingHttpResponse)

        assert response.status_code == HTTPStatus.OK
        assert response["Content-Type"] == "text/event-stream"
        response_iter = aiter(response)
        event1 = await anext(response_iter)
        event2 = await anext(response_iter)
        assert event1 == event2

    @django_4_2_plus
    async def test_success_template_change(self):
        response = await self.async_client.get("/__reload__/events/")
        assert isinstance(response, StreamingHttpResponse)
        views.should_reload_event.set()

        assert response.status_code == HTTPStatus.OK
        assert response["Content-Type"] == "text/event-stream"
        response_iter = aiter(response)
        # Skip version ID message
        await anext(response_iter)
        event = await anext(response_iter)
        assert event == b'data: {"type": "reload"}\n\n'
        assert not views.should_reload_event.is_set()

    @skipIf(django.VERSION >= (4, 2), "Requires Django < 4.2")
    async def test_fail_old_django(self):
        response = await self.async_client.get("/__reload__/events/")

        assert response.status_code == HTTPStatus.NOT_IMPLEMENTED
        assert response["content-type"] == "text/plain"
        assert response.content == b"ASGI requires Django 4.2+"



from __future__ import annotations

from django.core.exceptions import MiddlewareNotUsed
from django.http import HttpRequest
from django.http import HttpResponse
from django.http import StreamingHttpResponse
from django.http.response import HttpResponseBase
from django.test import override_settings
from django.test import RequestFactory
from django.test import SimpleTestCase

from django_browser_reload.middleware import BrowserReloadMiddleware


@override_settings(DEBUG=True)
class BrowserReloadMiddlewareTests(SimpleTestCase):
    request_factory = RequestFactory()

    def setUp(self):
        self.request = self.request_factory.get("/")
        self.response: HttpResponseBase = HttpResponse("<html><body></body></html>")

        def get_response(request: HttpRequest) -> HttpResponseBase:
            return self.response

        self.middleware = BrowserReloadMiddleware(get_response)

    @override_settings(DEBUG=False)
    def test_not_debug_instantiation(self):
        with self.assertRaises(MiddlewareNotUsed):
            BrowserReloadMiddleware(self.middleware.get_response)

    @override_settings(DEBUG=False)
    def test_not_debug(self):
        response = self.middleware(self.request)

        assert isinstance(response, HttpResponse)
        assert response.content == b"<html><body></body></html>"

    def test_streaming_response(self):
        # content that the middleware could inject in if it supported streaming
        # responses
        content_iter = iter(["<html><body>", "</body></html>"])
        self.response = StreamingHttpResponse(content_iter)

        response = self.middleware(self.request)

        assert isinstance(response, StreamingHttpResponse)
        content = b"".join(response)
        assert content == b"<html><body></body></html>"

    def test_encoded_response(self):
        self.response["Content-Encoding"] = "zabble"

        response = self.middleware(self.request)

        assert isinstance(response, HttpResponse)
        assert response.content == b"<html><body></body></html>"

    def test_text_response(self):
        self.response["Content-Type"] = "text/plain"

        response = self.middleware(self.request)

        assert isinstance(response, HttpResponse)
        assert response.content == b"<html><body></body></html>"

    def test_no_match(self):
        self.response = HttpResponse("<html><body>Woops")

        response = self.middleware(self.request)

        assert isinstance(response, HttpResponse)
        assert response.content == b"<html><body>Woops"

    def test_success(self):
        self.response = HttpResponse("<html><body></body></html>")
        self.response["Content-Length"] = len(self.response.content)

        response = self.middleware(self.request)

        assert isinstance(response, HttpResponse)
        assert response.content == (
            b"<html><body>"
            + b'<script src="/static/django-browser-reload/reload-listener.js"'
            + b' data-worker-script-path="/static/django-browser-reload/'
            + b'reload-worker.js"'
            + b' data-events-path="/__reload__/events/" defer></script>'
            + b"</body></html>"
        )

    async def test_async_success(self):
        async def get_response(request: HttpRequest) -> HttpResponse:
            return HttpResponse("<html><body></body></html>")

        middleware = BrowserReloadMiddleware(get_response)

        result = middleware(self.request)
        assert not isinstance(result, HttpResponseBase)
        response = await result

        assert isinstance(response, HttpResponse)
        assert response.content == (
            b"<html><body>"
            + b'<script src="/static/django-browser-reload/reload-listener.js"'
            + b' data-worker-script-path="/static/django-browser-reload/'
            + b'reload-worker.js"'
            + b' data-events-path="/__reload__/events/" defer></script>'
            + b"</body></html>"
        )

    def test_multipart_content_type(self):
        self.response["Content-Type"] = "text/html; thingy=that; charset=utf-8"

        response = self.middleware(self.request)

        assert isinstance(response, HttpResponse)
        assert response.content == (
            b"<html><body>"
            + b'<script src="/static/django-browser-reload/reload-listener.js"'
            + b' data-worker-script-path="/static/django-browser-reload/'
            + b'reload-worker.js"'
            + b' data-events-path="/__reload__/events/" defer></script>'
            + b"</body></html>"
        )

    def test_two_matches(self):
        self.response = HttpResponse("<html><body></body><body></body></html>")

        response = self.middleware(self.request)

        assert isinstance(response, HttpResponse)
        assert response.content == (
            b"<html><body></body><body>"
            + b'<script src="/static/django-browser-reload/reload-listener.js"'
            + b' data-worker-script-path="/static/django-browser-reload/'
            + b'reload-worker.js"'
            + b' data-events-path="/__reload__/events/" defer></script>'
            + b"</body></html>"
        )



from __future__ import annotations

from django.test import override_settings
from django.test import SimpleTestCase

from django_browser_reload.jinja import django_browser_reload_script


class DjangoBrowserReloadScriptTests(SimpleTestCase):
    def test_non_debug_empty(self):
        result = django_browser_reload_script()

        assert result == ""

    def test_debug_success(self):
        with override_settings(DEBUG=True):
            result = django_browser_reload_script()

        assert result == (
            '<script src="/static/django-browser-reload/reload-listener.js"'
            + ' data-worker-script-path="/static/django-browser-reload/'
            + 'reload-worker.js"'
            + ' data-events-path="/__reload__/events/" defer></script>'
        )
